@@ -1,7 +1,7 @@
 /*
  * GridTools
  *
- * Copyright (c) 2014-2019, ETH Zurich
+ * Copyright (c) 2014-2023, ETH Zurich
  * All rights reserved.
  *
  * Please, refer to the LICENSE file in the root directory.
@@ -10,7 +10,6 @@
 #pragma once
 
 #include "../../common/defs.hpp"
-#include "../../common/numerics.hpp"
 #include "descriptor_base.hpp"
 
 #ifdef GT_CUDACC
@@ -56,21 +55,20 @@
 #include <vector>
 
 #include "../../common/array.hpp"
-#include "../../common/make_array.hpp"
-#include "../../common/numerics.hpp"
 #include "../low_level/translate.hpp"
 #include "field_on_the_fly.hpp"
 #include "helpers_impl.hpp"
+#include "numerics.hpp"
 
 namespace gridtools {
     namespace gcl {
         template <typename HaloExch, typename proc_layout_abs>
         class hndlr_generic<HaloExch, proc_layout_abs, cpu> : public descriptor_base<HaloExch> {
-            static const int DIMS = 3;
-            array<char *, static_pow3(DIMS)> send_buffer; // One entry will not be used...
-            array<char *, static_pow3(DIMS)> recv_buffer;
-            array<int, static_pow3(DIMS)> send_buffer_size; // One entry will not be used...
-            array<int, static_pow3(DIMS)> recv_buffer_size;
+            using DIMS_t = std::integral_constant<int, 3>;
+            array<char *, static_pow3(DIMS_t::value)> send_buffer; // One entry will not be used...
+            array<char *, static_pow3(DIMS_t::value)> recv_buffer;
+            array<int, static_pow3(DIMS_t::value)> send_buffer_size; // One entry will not be used...
+            array<int, static_pow3(DIMS_t::value)> recv_buffer_size;
 
           public:
             typedef descriptor_base<HaloExch> base_type;
@@ -87,7 +85,7 @@ namespace gridtools {
             /**
                Type of the translation used to map dimensions to buffer addresses
              */
-            typedef translate_t<DIMS, typename default_layout_map<DIMS>::type> translate;
+            typedef translate_t<DIMS_t::value> translate;
 
             hndlr_generic(grid_type const &g)
                 : base_type(g), send_buffer{nullptr}, recv_buffer{nullptr}, send_buffer_size{0}, recv_buffer_size{0} {}
@@ -121,7 +119,7 @@ namespace gridtools {
                 int typesize = sizeof(DataType)) {
 
                 typedef typename field_on_the_fly<DataType, f_layoutmap, traits>::inner_layoutmap t_layoutmap;
-                array<int, DIMS> eta;
+                array<int, DIMS_t::value> eta;
                 for (int i = -1; i <= 1; ++i) {
                     for (int j = -1; j <= 1; ++j) {
                         for (int k = -1; k <= 1; ++k) {
@@ -142,10 +140,10 @@ namespace gridtools {
                                 recv_buffer[translate()(i, j, k)] =
                                     gcl_alloc<char, arch_type>::alloc(recv_buffer_size[translate()(i, j, k)]);
 
-                                typedef typename layout_transform<t_layoutmap, proc_layout_abs>::type proc_layout;
-                                const int i_P = make_array(i, j, k)[proc_layout::at(0)];
-                                const int j_P = make_array(i, j, k)[proc_layout::at(1)];
-                                const int k_P = make_array(i, j, k)[proc_layout::at(2)];
+                                using proc_layout = layout_transform<t_layoutmap, proc_layout_abs>;
+                                const int i_P = nth<proc_layout, 0>(i, j, k);
+                                const int j_P = nth<proc_layout, 1>(i, j, k);
+                                const int k_P = nth<proc_layout, 2>(i, j, k);
 
                                 base_type::m_haloexch.register_send_to_buffer(&(send_buffer[translate()(i, j, k)][0]),
                                     send_buffer_size[translate()(i, j, k)],
@@ -176,7 +174,7 @@ namespace gridtools {
                halos.
              */
             template <typename DataType, typename t_layoutmap>
-            void setup(array<size_t, static_pow3(DIMS)> const &buffer_size_list) {
+            void setup(array<size_t, static_pow3(DIMS_t::value)> const &buffer_size_list) {
                 for (int i = -1; i <= 1; ++i) {
                     for (int j = -1; j <= 1; ++j) {
                         for (int k = -1; k <= 1; ++k) {
@@ -188,10 +186,10 @@ namespace gridtools {
                                 send_buffer_size[translate()(i, j, k)] = (buffer_size_list[translate()(i, j, k)]);
                                 recv_buffer_size[translate()(i, j, k)] = (buffer_size_list[translate()(i, j, k)]);
 
-                                typedef typename layout_transform<t_layoutmap, proc_layout_abs>::type proc_layout;
-                                const int i_P = make_array(i, j, k)[proc_layout::at(0)];
-                                const int j_P = make_array(i, j, k)[proc_layout::at(1)];
-                                const int k_P = make_array(i, j, k)[proc_layout::at(2)];
+                                using proc_layout = layout_transform<t_layoutmap, proc_layout_abs>;
+                                const int i_P = nth<proc_layout, 0>(i, j, k);
+                                const int j_P = nth<proc_layout, 1>(i, j, k);
+                                const int k_P = nth<proc_layout, 2>(i, j, k);
 
                                 base_type::m_haloexch.register_send_to_buffer(&(send_buffer[translate()(i, j, k)][0]),
                                     buffer_size_list[translate()(i, j, k)],
@@ -212,24 +210,24 @@ namespace gridtools {
             }
 
             template <typename... FIELDS>
-            void pack(const FIELDS &... _fields) const {
+            void pack(const FIELDS &..._fields) const {
                 for (int ii = -1; ii <= 1; ++ii) {
                     for (int jj = -1; jj <= 1; ++jj) {
                         for (int kk = -1; kk <= 1; ++kk) {
                             char *it = reinterpret_cast<char *>(&(send_buffer[translate()(ii, jj, kk)][0]));
-                            pack_dims<DIMS, 0>()(*this, ii, jj, kk, it, _fields...);
+                            pack_dims<DIMS_t::value, 0>()(*this, ii, jj, kk, it, _fields...);
                         }
                     }
                 }
             }
 
             template <typename... FIELDS>
-            void unpack(const FIELDS &... _fields) const {
+            void unpack(const FIELDS &..._fields) const {
                 for (int ii = -1; ii <= 1; ++ii) {
                     for (int jj = -1; jj <= 1; ++jj) {
                         for (int kk = -1; kk <= 1; ++kk) {
                             char *it = reinterpret_cast<char *>(&(recv_buffer[translate()(ii, jj, kk)][0]));
-                            unpack_dims<DIMS, 0>()(*this, ii, jj, kk, it, _fields...);
+                            unpack_dims<DIMS_t::value, 0>()(*this, ii, jj, kk, it, _fields...);
                         }
                     }
                 }
@@ -249,7 +247,7 @@ namespace gridtools {
                             typename field_on_the_fly<T1, T2, T3>::value_type *it =
                                 reinterpret_cast<typename field_on_the_fly<T1, T2, T3>::value_type *>(
                                     &(send_buffer[translate()(ii, jj, kk)][0]));
-                            pack_vector_dims<DIMS, 0>()(*this, ii, jj, kk, it, fields);
+                            pack_vector_dims<DIMS_t::value, 0>()(*this, ii, jj, kk, it, fields);
                         }
                     }
                 }
@@ -269,7 +267,7 @@ namespace gridtools {
                             typename field_on_the_fly<T1, T2, T3>::value_type *it =
                                 reinterpret_cast<typename field_on_the_fly<T1, T2, T3>::value_type *>(
                                     &(recv_buffer[translate()(ii, jj, kk)][0]));
-                            unpack_vector_dims<DIMS, 0>()(*this, ii, jj, kk, it, fields);
+                            unpack_vector_dims<DIMS_t::value, 0>()(*this, ii, jj, kk, it, fields);
                         }
                     }
                 }
@@ -287,15 +285,14 @@ namespace gridtools {
 
                 template <typename T, typename iterator, typename FIRST, typename... FIELDS>
                 void operator()(
-                    const T &hm, int ii, int jj, int kk, iterator &it, FIRST const &first, const FIELDS &... _fields)
+                    const T &hm, int ii, int jj, int kk, iterator &it, FIRST const &first, const FIELDS &..._fields)
                     const {
-                    typedef
-                        typename layout_transform<typename FIRST::inner_layoutmap, proc_layout_abs>::type proc_layout;
-                    const int ii_P = make_array(ii, jj, kk)[proc_layout::at(0)];
-                    const int jj_P = make_array(ii, jj, kk)[proc_layout::at(1)];
-                    const int kk_P = make_array(ii, jj, kk)[proc_layout::at(2)];
+                    using proc_layout = layout_transform<typename FIRST::inner_layoutmap, proc_layout_abs>;
+                    const int ii_P = nth<proc_layout, 0>(ii, jj, kk);
+                    const int jj_P = nth<proc_layout, 1>(ii, jj, kk);
+                    const int kk_P = nth<proc_layout, 2>(ii, jj, kk);
                     if ((ii != 0 || jj != 0 || kk != 0) && (hm.pattern().proc_grid().proc(ii_P, jj_P, kk_P) != -1)) {
-                        first.pack(make_array(ii, jj, kk), first.ptr, it);
+                        first.pack({ii, jj, kk}, first.ptr, it);
                         operator()(hm, ii, jj, kk, it, _fields...);
                     }
                 }
@@ -312,15 +309,14 @@ namespace gridtools {
 
                 template <typename T, typename iterator, typename FIRST, typename... FIELDS>
                 void operator()(
-                    const T &hm, int ii, int jj, int kk, iterator &it, FIRST const &first, const FIELDS &... _fields)
+                    const T &hm, int ii, int jj, int kk, iterator &it, FIRST const &first, const FIELDS &..._fields)
                     const {
-                    typedef
-                        typename layout_transform<typename FIRST::inner_layoutmap, proc_layout_abs>::type proc_layout;
-                    const int ii_P = make_array(ii, jj, kk)[proc_layout::at(0)];
-                    const int jj_P = make_array(ii, jj, kk)[proc_layout::at(1)];
-                    const int kk_P = make_array(ii, jj, kk)[proc_layout::at(2)];
+                    using proc_layout = layout_transform<typename FIRST::inner_layoutmap, proc_layout_abs>;
+                    const int ii_P = nth<proc_layout, 0>(ii, jj, kk);
+                    const int jj_P = nth<proc_layout, 1>(ii, jj, kk);
+                    const int kk_P = nth<proc_layout, 2>(ii, jj, kk);
                     if ((ii != 0 || jj != 0 || kk != 0) && (hm.pattern().proc_grid().proc(ii_P, jj_P, kk_P) != -1)) {
-                        first.unpack(make_array(ii, jj, kk), first.ptr, it);
+                        first.unpack({ii, jj, kk}, first.ptr, it);
                         operator()(hm, ii, jj, kk, it, _fields...);
                     }
                 }
@@ -334,14 +330,14 @@ namespace gridtools {
 
                 template <typename T, typename iterator, typename array_of_fotf>
                 void operator()(const T &hm, int ii, int jj, int kk, iterator &it, array_of_fotf const &_fields) const {
-                    typedef typename layout_transform<typename array_of_fotf::value_type::inner_layoutmap,
-                        proc_layout_abs>::type proc_layout;
-                    const int ii_P = make_array(ii, jj, kk)[proc_layout::at(0)];
-                    const int jj_P = make_array(ii, jj, kk)[proc_layout::at(1)];
-                    const int kk_P = make_array(ii, jj, kk)[proc_layout::at(2)];
+                    using proc_layout =
+                        layout_transform<typename array_of_fotf::value_type::inner_layoutmap, proc_layout_abs>;
+                    const int ii_P = nth<proc_layout, 0>(ii, jj, kk);
+                    const int jj_P = nth<proc_layout, 1>(ii, jj, kk);
+                    const int kk_P = nth<proc_layout, 2>(ii, jj, kk);
                     if ((ii != 0 || jj != 0 || kk != 0) && (hm.pattern().proc_grid().proc(ii_P, jj_P, kk_P) != -1)) {
                         for (unsigned int fi = 0; fi < _fields.size(); ++fi) {
-                            _fields[fi].pack(make_array(ii, jj, kk), _fields[fi].ptr, it);
+                            _fields[fi].pack({ii, jj, kk}, _fields[fi].ptr, it);
                         }
                     }
                 }
@@ -355,14 +351,14 @@ namespace gridtools {
 
                 template <typename T, typename iterator, typename array_of_fotf>
                 void operator()(const T &hm, int ii, int jj, int kk, iterator &it, array_of_fotf const &_fields) const {
-                    typedef typename layout_transform<typename array_of_fotf::value_type::inner_layoutmap,
-                        proc_layout_abs>::type proc_layout;
-                    const int ii_P = make_array(ii, jj, kk)[proc_layout::at(0)];
-                    const int jj_P = make_array(ii, jj, kk)[proc_layout::at(1)];
-                    const int kk_P = make_array(ii, jj, kk)[proc_layout::at(2)];
+                    using proc_layout =
+                        layout_transform<typename array_of_fotf::value_type::inner_layoutmap, proc_layout_abs>;
+                    const int ii_P = nth<proc_layout, 0>(ii, jj, kk);
+                    const int jj_P = nth<proc_layout, 1>(ii, jj, kk);
+                    const int kk_P = nth<proc_layout, 2>(ii, jj, kk);
                     if ((ii != 0 || jj != 0 || kk != 0) && (hm.pattern().proc_grid().proc(ii_P, jj_P, kk_P) != -1)) {
                         for (unsigned int fi = 0; fi < _fields.size(); ++fi) {
-                            _fields[fi].unpack(make_array(ii, jj, kk), _fields[fi].ptr, it);
+                            _fields[fi].unpack({ii, jj, kk}, _fields[fi].ptr, it);
                         }
                     }
                 }
@@ -374,18 +370,18 @@ namespace gridtools {
         class hndlr_generic<HaloExch, proc_layout_abs, gpu> : public descriptor_base<HaloExch> {
             typedef gpu arch_type;
 
-            static const int DIMS = 3;
-            array<char *, static_pow3(DIMS)> send_buffer; // One entry will not be used...
-            array<char *, static_pow3(DIMS)> recv_buffer;
-            array<int, static_pow3(DIMS)> send_buffer_size; // One entry will not be used...
-            array<int, static_pow3(DIMS)> recv_buffer_size;
+            using DIMS_t = std::integral_constant<int, 3>;
+            array<char *, static_pow3(DIMS_t::value)> send_buffer; // One entry will not be used...
+            array<char *, static_pow3(DIMS_t::value)> recv_buffer;
+            array<int, static_pow3(DIMS_t::value)> send_buffer_size; // One entry will not be used...
+            array<int, static_pow3(DIMS_t::value)> recv_buffer_size;
             char **d_send_buffer;
             char **d_recv_buffer;
 
             int *prefix_send_size;
             int *prefix_recv_size;
-            array<int, static_pow3(DIMS)> send_size;
-            array<int, static_pow3(DIMS)> recv_size;
+            array<int, static_pow3(DIMS_t::value)> send_size;
+            array<int, static_pow3(DIMS_t::value)> recv_size;
 
             int *d_send_size;
             int *d_recv_size;
@@ -405,7 +401,7 @@ namespace gridtools {
             /**
                Type of the translation used to map dimensions to buffer addresses
              */
-            typedef translate_t<DIMS, typename default_layout_map<DIMS>::type> translate;
+            typedef translate_t<DIMS_t::value> translate;
 
             hndlr_generic(grid_type const &g)
                 : base_type(g), send_buffer{nullptr}, recv_buffer{nullptr}, send_buffer_size{0}, recv_buffer_size{0} {}
@@ -458,15 +454,14 @@ namespace gridtools {
                     for (int jj = -1; jj <= 1; ++jj)
                         for (int kk = -1; kk <= 1; ++kk)
                             if (ii != 0 || jj != 0 || kk != 0) {
-                                typedef typename layout_transform<data_layout, proc_layout_abs>::type map_type;
+                                using map_type = layout_transform<data_layout, proc_layout_abs>;
 
-                                const int ii_P = make_array(ii, jj, kk)[map_type::at(0)];
-                                const int jj_P = make_array(ii, jj, kk)[map_type::at(1)];
-                                const int kk_P = make_array(ii, jj, kk)[map_type::at(2)];
+                                const int ii_P = nth<map_type, 0>(ii, jj, kk);
+                                const int jj_P = nth<map_type, 1>(ii, jj, kk);
+                                const int kk_P = nth<map_type, 2>(ii, jj, kk);
 
                                 if (base_type::pattern().proc_grid().proc(ii_P, jj_P, kk_P) != -1) {
-                                    send_size[translate()(ii, jj, kk)] =
-                                        halo_example.send_buffer_size(make_array(ii, jj, kk));
+                                    send_size[translate()(ii, jj, kk)] = halo_example.send_buffer_size({ii, jj, kk});
 
                                     send_buffer[translate()(ii, jj, kk)] = gcl_alloc<char, arch_type>::alloc(
                                         send_size[translate()(ii, jj, kk)] * max_fields_n * typesize);
@@ -478,8 +473,7 @@ namespace gridtools {
                                         jj_P,
                                         kk_P);
 
-                                    recv_size[translate()(ii, jj, kk)] =
-                                        halo_example.recv_buffer_size(make_array(ii, jj, kk));
+                                    recv_size[translate()(ii, jj, kk)] = halo_example.recv_buffer_size({ii, jj, kk});
 
                                     recv_buffer[translate()(ii, jj, kk)] = gcl_alloc<char, arch_type>::alloc(
                                         recv_size[translate()(ii, jj, kk)] * max_fields_n * typesize);
@@ -508,15 +502,19 @@ namespace gridtools {
                                 }
                             }
 
-                GT_CUDA_CHECK(cudaMalloc(&d_send_buffer, static_pow3(DIMS) * sizeof(DataType *)));
+                GT_CUDA_CHECK(cudaMalloc(&d_send_buffer, static_pow3(DIMS_t::value) * sizeof(DataType *)));
 
-                GT_CUDA_CHECK(cudaMemcpy(
-                    d_send_buffer, &send_buffer[0], static_pow3(DIMS) * sizeof(DataType *), cudaMemcpyHostToDevice));
+                GT_CUDA_CHECK(cudaMemcpy(d_send_buffer,
+                    &send_buffer[0],
+                    static_pow3(DIMS_t::value) * sizeof(DataType *),
+                    cudaMemcpyHostToDevice));
 
-                GT_CUDA_CHECK(cudaMalloc(&d_recv_buffer, static_pow3(DIMS) * sizeof(DataType *)));
+                GT_CUDA_CHECK(cudaMalloc(&d_recv_buffer, static_pow3(DIMS_t::value) * sizeof(DataType *)));
 
-                GT_CUDA_CHECK(cudaMemcpy(
-                    d_recv_buffer, &recv_buffer[0], static_pow3(DIMS) * sizeof(DataType *), cudaMemcpyHostToDevice));
+                GT_CUDA_CHECK(cudaMemcpy(d_recv_buffer,
+                    &recv_buffer[0],
+                    static_pow3(DIMS_t::value) * sizeof(DataType *),
+                    cudaMemcpyHostToDevice));
             }
 
             /**
@@ -527,8 +525,8 @@ namespace gridtools {
             template <typename T1, typename T2, template <typename> class T3>
             void pack(std::vector<field_on_the_fly<T1, T2, T3>> const &_fields) {
 
-                typedef typename layout_transform<typename field_on_the_fly<T1, T2, T3>::inner_layoutmap,
-                    proc_layout_abs>::type map_type;
+                using map_type =
+                    layout_transform<typename field_on_the_fly<T1, T2, T3>::inner_layoutmap, proc_layout_abs>;
 
                 std::vector<field_on_the_fly<T1, T2, T3>> fields = _fields;
 
@@ -536,9 +534,9 @@ namespace gridtools {
                     int ii = 1;
                     int jj = 0;
                     int kk = 0;
-                    const int ii_P = make_array(ii, jj, kk)[map_type::at(0)];
-                    const int jj_P = make_array(ii, jj, kk)[map_type::at(1)];
-                    const int kk_P = make_array(ii, jj, kk)[map_type::at(2)];
+                    const int ii_P = nth<map_type, 0>(ii, jj, kk);
+                    const int jj_P = nth<map_type, 1>(ii, jj, kk);
+                    const int kk_P = nth<map_type, 2>(ii, jj, kk);
                     if ((base_type::pattern().proc_grid().proc(ii_P, jj_P, kk_P) == -1)) {
                         for (int l = 0; l < fields.size(); ++l)
                             fields[l].halos[0].reset_minus();
@@ -548,9 +546,9 @@ namespace gridtools {
                     int ii = -1;
                     int jj = 0;
                     int kk = 0;
-                    const int ii_P = make_array(ii, jj, kk)[map_type::at(0)];
-                    const int jj_P = make_array(ii, jj, kk)[map_type::at(1)];
-                    const int kk_P = make_array(ii, jj, kk)[map_type::at(2)];
+                    const int ii_P = nth<map_type, 0>(ii, jj, kk);
+                    const int jj_P = nth<map_type, 1>(ii, jj, kk);
+                    const int kk_P = nth<map_type, 2>(ii, jj, kk);
                     if ((base_type::pattern().proc_grid().proc(ii_P, jj_P, kk_P) == -1)) {
                         for (int l = 0; l < fields.size(); ++l)
                             fields[l].halos[0].reset_plus();
@@ -560,9 +558,9 @@ namespace gridtools {
                     int ii = 0;
                     int jj = 1;
                     int kk = 0;
-                    const int ii_P = make_array(ii, jj, kk)[map_type::at(0)];
-                    const int jj_P = make_array(ii, jj, kk)[map_type::at(1)];
-                    const int kk_P = make_array(ii, jj, kk)[map_type::at(2)];
+                    const int ii_P = nth<map_type, 0>(ii, jj, kk);
+                    const int jj_P = nth<map_type, 1>(ii, jj, kk);
+                    const int kk_P = nth<map_type, 2>(ii, jj, kk);
                     if ((base_type::pattern().proc_grid().proc(ii_P, jj_P, kk_P) == -1)) {
                         for (int l = 0; l < fields.size(); ++l)
                             fields[l].halos[1].reset_minus();
@@ -572,9 +570,9 @@ namespace gridtools {
                     int ii = 0;
                     int jj = -1;
                     int kk = 0;
-                    const int ii_P = make_array(ii, jj, kk)[map_type::at(0)];
-                    const int jj_P = make_array(ii, jj, kk)[map_type::at(1)];
-                    const int kk_P = make_array(ii, jj, kk)[map_type::at(2)];
+                    const int ii_P = nth<map_type, 0>(ii, jj, kk);
+                    const int jj_P = nth<map_type, 1>(ii, jj, kk);
+                    const int kk_P = nth<map_type, 2>(ii, jj, kk);
                     if ((base_type::pattern().proc_grid().proc(ii_P, jj_P, kk_P) == -1)) {
                         for (int l = 0; l < fields.size(); ++l)
                             fields[l].halos[1].reset_plus();
@@ -584,9 +582,9 @@ namespace gridtools {
                     int ii = 0;
                     int jj = 0;
                     int kk = 1;
-                    const int ii_P = make_array(ii, jj, kk)[map_type::at(0)];
-                    const int jj_P = make_array(ii, jj, kk)[map_type::at(1)];
-                    const int kk_P = make_array(ii, jj, kk)[map_type::at(2)];
+                    const int ii_P = nth<map_type, 0>(ii, jj, kk);
+                    const int jj_P = nth<map_type, 1>(ii, jj, kk);
+                    const int kk_P = nth<map_type, 2>(ii, jj, kk);
                     if ((base_type::pattern().proc_grid().proc(ii_P, jj_P, kk_P) == -1)) {
                         for (int l = 0; l < fields.size(); ++l)
                             fields[l].halos[2].reset_minus();
@@ -596,9 +594,9 @@ namespace gridtools {
                     int ii = 0;
                     int jj = 0;
                     int kk = -1;
-                    const int ii_P = make_array(ii, jj, kk)[map_type::at(0)];
-                    const int jj_P = make_array(ii, jj, kk)[map_type::at(1)];
-                    const int kk_P = make_array(ii, jj, kk)[map_type::at(2)];
+                    const int ii_P = nth<map_type, 0>(ii, jj, kk);
+                    const int jj_P = nth<map_type, 1>(ii, jj, kk);
+                    const int kk_P = nth<map_type, 2>(ii, jj, kk);
                     if ((base_type::pattern().proc_grid().proc(ii_P, jj_P, kk_P) == -1)) {
                         for (int l = 0; l < fields.size(); ++l)
                             fields[l].halos[2].reset_plus();
@@ -610,16 +608,16 @@ namespace gridtools {
                 for (int ii = -1; ii <= 1; ++ii)
                     for (int jj = -1; jj <= 1; ++jj)
                         for (int kk = -1; kk <= 1; ++kk) {
-                            const int ii_P = make_array(ii, jj, kk)[map_type::at(0)];
-                            const int jj_P = make_array(ii, jj, kk)[map_type::at(1)];
-                            const int kk_P = make_array(ii, jj, kk)[map_type::at(2)];
+                            const int ii_P = nth<map_type, 0>(ii, jj, kk);
+                            const int jj_P = nth<map_type, 1>(ii, jj, kk);
+                            const int kk_P = nth<map_type, 2>(ii, jj, kk);
                             if ((base_type::pattern().proc_grid().proc(ii_P, jj_P, kk_P) != -1)) {
                                 if (ii != 0 || jj != 0 || kk != 0) {
                                     prefix_send_size[0 + translate()(ii, jj, kk)] = 0;
                                     for (int l = 1; l < fields.size(); ++l) {
                                         prefix_send_size[l * 27 + translate()(ii, jj, kk)] =
                                             prefix_send_size[(l - 1) * 27 + translate()(ii, jj, kk)] +
-                                            fields[l - 1].send_buffer_size(make_array(ii, jj, kk));
+                                            fields[l - 1].send_buffer_size({ii, jj, kk});
                                     }
                                 }
                             }
@@ -666,8 +664,8 @@ namespace gridtools {
             */
             template <typename T1, typename T2, template <typename> class T3>
             void unpack(std::vector<field_on_the_fly<T1, T2, T3>> const &_fields) {
-                typedef typename layout_transform<typename field_on_the_fly<T1, T2, T3>::inner_layoutmap,
-                    proc_layout_abs>::type map_type;
+                using map_type =
+                    layout_transform<typename field_on_the_fly<T1, T2, T3>::inner_layoutmap, proc_layout_abs>;
 
                 std::vector<field_on_the_fly<T1, T2, T3>> fields = _fields;
 
@@ -675,9 +673,9 @@ namespace gridtools {
                     int ii = 1;
                     int jj = 0;
                     int kk = 0;
-                    const int ii_P = make_array(ii, jj, kk)[map_type::at(0)];
-                    const int jj_P = make_array(ii, jj, kk)[map_type::at(1)];
-                    const int kk_P = make_array(ii, jj, kk)[map_type::at(2)];
+                    const int ii_P = nth<map_type, 0>(ii, jj, kk);
+                    const int jj_P = nth<map_type, 1>(ii, jj, kk);
+                    const int kk_P = nth<map_type, 2>(ii, jj, kk);
                     if ((base_type::pattern().proc_grid().proc(ii_P, jj_P, kk_P) == -1)) {
                         for (int l = 0; l < fields.size(); ++l)
                             fields[l].halos[0].reset_plus();
@@ -687,9 +685,9 @@ namespace gridtools {
                     int ii = -1;
                     int jj = 0;
                     int kk = 0;
-                    const int ii_P = make_array(ii, jj, kk)[map_type::at(0)];
-                    const int jj_P = make_array(ii, jj, kk)[map_type::at(1)];
-                    const int kk_P = make_array(ii, jj, kk)[map_type::at(2)];
+                    const int ii_P = nth<map_type, 0>(ii, jj, kk);
+                    const int jj_P = nth<map_type, 1>(ii, jj, kk);
+                    const int kk_P = nth<map_type, 2>(ii, jj, kk);
                     if ((base_type::pattern().proc_grid().proc(ii_P, jj_P, kk_P) == -1)) {
                         for (int l = 0; l < fields.size(); ++l)
                             fields[l].halos[0].reset_minus();
@@ -699,9 +697,9 @@ namespace gridtools {
                     int ii = 0;
                     int jj = 1;
                     int kk = 0;
-                    const int ii_P = make_array(ii, jj, kk)[map_type::at(0)];
-                    const int jj_P = make_array(ii, jj, kk)[map_type::at(1)];
-                    const int kk_P = make_array(ii, jj, kk)[map_type::at(2)];
+                    const int ii_P = nth<map_type, 0>(ii, jj, kk);
+                    const int jj_P = nth<map_type, 1>(ii, jj, kk);
+                    const int kk_P = nth<map_type, 2>(ii, jj, kk);
                     if ((base_type::pattern().proc_grid().proc(ii_P, jj_P, kk_P) == -1)) {
                         for (int l = 0; l < fields.size(); ++l)
                             fields[l].halos[1].reset_plus();
@@ -711,9 +709,9 @@ namespace gridtools {
                     int ii = 0;
                     int jj = -1;
                     int kk = 0;
-                    const int ii_P = make_array(ii, jj, kk)[map_type::at(0)];
-                    const int jj_P = make_array(ii, jj, kk)[map_type::at(1)];
-                    const int kk_P = make_array(ii, jj, kk)[map_type::at(2)];
+                    const int ii_P = nth<map_type, 0>(ii, jj, kk);
+                    const int jj_P = nth<map_type, 1>(ii, jj, kk);
+                    const int kk_P = nth<map_type, 2>(ii, jj, kk);
                     if ((base_type::pattern().proc_grid().proc(ii_P, jj_P, kk_P) == -1)) {
                         for (int l = 0; l < fields.size(); ++l)
                             fields[l].halos[1].reset_minus();
@@ -723,9 +721,9 @@ namespace gridtools {
                     int ii = 0;
                     int jj = 0;
                     int kk = 1;
-                    const int ii_P = make_array(ii, jj, kk)[map_type::at(0)];
-                    const int jj_P = make_array(ii, jj, kk)[map_type::at(1)];
-                    const int kk_P = make_array(ii, jj, kk)[map_type::at(2)];
+                    const int ii_P = nth<map_type, 0>(ii, jj, kk);
+                    const int jj_P = nth<map_type, 1>(ii, jj, kk);
+                    const int kk_P = nth<map_type, 2>(ii, jj, kk);
                     if ((base_type::pattern().proc_grid().proc(ii_P, jj_P, kk_P) == -1)) {
                         for (int l = 0; l < fields.size(); ++l)
                             fields[l].halos[2].reset_plus();
@@ -735,9 +733,9 @@ namespace gridtools {
                     int ii = 0;
                     int jj = 0;
                     int kk = -1;
-                    const int ii_P = make_array(ii, jj, kk)[map_type::at(0)];
-                    const int jj_P = make_array(ii, jj, kk)[map_type::at(1)];
-                    const int kk_P = make_array(ii, jj, kk)[map_type::at(2)];
+                    const int ii_P = nth<map_type, 0>(ii, jj, kk);
+                    const int jj_P = nth<map_type, 1>(ii, jj, kk);
+                    const int kk_P = nth<map_type, 2>(ii, jj, kk);
                     if ((base_type::pattern().proc_grid().proc(ii_P, jj_P, kk_P) == -1)) {
                         for (int l = 0; l < fields.size(); ++l)
                             fields[l].halos[2].reset_minus();
@@ -747,22 +745,21 @@ namespace gridtools {
                 for (int ii = -1; ii <= 1; ++ii)
                     for (int jj = -1; jj <= 1; ++jj)
                         for (int kk = -1; kk <= 1; ++kk) {
-                            const int ii_P = make_array(ii, jj, kk)[map_type::at(0)];
-                            const int jj_P = make_array(ii, jj, kk)[map_type::at(1)];
-                            const int kk_P = make_array(ii, jj, kk)[map_type::at(2)];
+                            const int ii_P = nth<map_type, 0>(ii, jj, kk);
+                            const int jj_P = nth<map_type, 1>(ii, jj, kk);
+                            const int kk_P = nth<map_type, 2>(ii, jj, kk);
                             if ((base_type::pattern().proc_grid().proc(ii_P, jj_P, kk_P) != -1)) {
                                 if (ii != 0 || jj != 0 || kk != 0) {
                                     prefix_recv_size[0 + translate()(ii, jj, kk)] = 0;
                                     for (int l = 1; l < fields.size(); ++l) {
                                         prefix_recv_size[l * 27 + translate()(ii, jj, kk)] =
                                             prefix_recv_size[(l - 1) * 27 + translate()(ii, jj, kk)] +
-                                            fields[l - 1].recv_buffer_size(make_array(ii, jj, kk));
+                                            fields[l - 1].recv_buffer_size({ii, jj, kk});
                                     }
                                 }
                             }
                         }
 
-                // typedef translate_t<3,default_layout_map<3>::type > translate;
                 if (recv_size[translate()(0, 0, -1)]) {
                     m_unpackZL_generic(fields,
                         reinterpret_cast<typename field_on_the_fly<T1, T2, T3>::value_type **>(d_recv_buffer),
