@@ -1,7 +1,7 @@
 /*
  * GridTools
  *
- * Copyright (c) 2014-2019, ETH Zurich
+ * Copyright (c) 2014-2023, ETH Zurich
  * All rights reserved.
  *
  * Please, refer to the LICENSE file in the root directory.
@@ -28,9 +28,9 @@ namespace gridtools {
          */
         namespace lazy {
             template <class List>
-            using dedup = lfold<dedup_step_impl, typename clear<List>::type, List>;
+            using dedup = foldl<dedup_step_impl, typename clear<List>::type, List>;
         }
         template <class List>
-        using dedup = typename lazy::lfold<dedup_step_impl, typename lazy::clear<List>::type, List>::type;
+        using dedup = typename lazy::foldl<dedup_step_impl, typename lazy::clear<List>::type, List>::type;
     } // namespace meta
 } // namespace gridtools
