@@ -1,7 +1,7 @@
 /*
  * GridTools
  *
- * Copyright (c) 2014-2019, ETH Zurich
+ * Copyright (c) 2014-2023, ETH Zurich
  * All rights reserved.
  *
  * Please, refer to the LICENSE file in the root directory.
@@ -10,13 +10,14 @@
 
 #pragma once
 
+#include <type_traits>
+
 #include "length.hpp"
 #include "macros.hpp"
-#include "type_traits.hpp"
 
 namespace gridtools {
     namespace meta {
         template <class T>
-        using is_empty = bool_constant<length<T>::value == 0>;
+        using is_empty = std::bool_constant<length<T>::value == 0>;
     } // namespace meta
 } // namespace gridtools
