@@ -1,7 +1,7 @@
 /*
  * GridTools
  *
- * Copyright (c) 2014-2019, ETH Zurich
+ * Copyright (c) 2014-2023, ETH Zurich
  * All rights reserved.
  *
  * Please, refer to the LICENSE file in the root directory.
@@ -11,6 +11,7 @@
 
 #include <limits>
 #include <type_traits>
+#include <utility>
 
 #include "../common/defs.hpp"
 #include "../common/host_device.hpp"
@@ -22,7 +23,6 @@
 #include "../meta/logical.hpp"
 #include "../meta/macros.hpp"
 #include "../meta/push_front.hpp"
-#include "../meta/type_traits.hpp"
 #include "simple_ptr_holder.hpp"
 
 /**
@@ -167,36 +167,10 @@ namespace gridtools {
             struct get_static_const_value;
             template <class T>
             struct get_static_const_value<T,
-                std::enable_if_t<std::is_integral<decltype(T::value)>::value &&
-                                 std::is_const<decltype(T::value)>::value &&
-                                 !std::is_member_pointer<decltype(&T::value)>::value>>
+                std::enable_if_t<std::is_integral_v<decltype(T::value)> && std::is_const_v<decltype(T::value)> &&
+                                 !std::is_member_pointer_v<decltype(&T::value)>>>
                 : std::integral_constant<decltype(T::value), T::value> {};
 
-            /**
-             *  generic trait for integral constant
-             *
-             *  TODO(anstaf) : consider moving it to `meta` or to `common`
-             */
-            template <class T, class = void>
-            struct is_integral_constant : std::false_type {};
-            template <class T>
-            struct is_integral_constant<T,
-                std::enable_if_t<std::is_empty<T>::value && std::is_integral<decltype(T::value)>::value &&
-                                 std::is_convertible<T, typename get_static_const_value<T>::value_type>::value &&
-                                 T() == get_static_const_value<T>::value>> : std::true_type {};
-
-            /**
-             *  generic trait for integral constant of Val
-             *
-             *  TODO(anstaf) : consider moving it to `meta` or to `common`
-             */
-            template <class T, int Val, class = void>
-            struct is_integral_constant_of : std::false_type {};
-
-            template <class T, int Val>
-            struct is_integral_constant_of<T, Val, std::enable_if_t<is_integral_constant<T>::value && T() == Val>>
-                : std::true_type {};
-
             /////// BEGIN defaults PART /////
 
             template <class Ptr>
@@ -206,21 +180,12 @@ namespace gridtools {
             using default_ptr_diff = decltype(::gridtools::sid::concept_impl_::sid_get_default_ptr_diff(
                 std::declval<std::add_lvalue_reference_t<std::add_const_t<Ptr>>>()));
 
-            template <class T, class = void>
-            struct is_empty_or_tuple_of_empties : std::is_empty<T> {};
-
-            template <class Tup, class Types = tuple_util::traits::to_types<Tup>>
-            using is_tuple_of_empties = meta::all_of<is_empty_or_tuple_of_empties, Types>;
-
-            template <class Tup>
-            struct is_empty_or_tuple_of_empties<Tup, std::enable_if_t<is_tuple_of_empties<Tup>::value>>
-                : std::true_type {};
-
             namespace lazy {
                 template <class, class = void>
                 struct default_kind;
                 template <class T>
-                struct default_kind<T, std::enable_if_t<is_empty_or_tuple_of_empties<std::decay_t<T>>::value>>
+                struct default_kind<T,
+                    std::enable_if_t<tuple_util::is_empty_or_tuple_of_empties<std::decay_t<T>>::value>>
                     : std::decay<T> {};
             } // namespace lazy
             GT_META_DELEGATE_TO_LAZY(default_kind, class T, T);
@@ -250,7 +215,7 @@ namespace gridtools {
              *  C-array specialization
              */
             template <class T, class Res = std::add_pointer_t<std::remove_all_extents_t<T>>>
-            std::enable_if_t<std::is_array<T>::value, host_device::simple_ptr_holder<Res>> get_origin(T &obj) {
+            std::enable_if_t<std::is_array_v<T>, host_device::simple_ptr_holder<Res>> get_origin(T &obj) {
                 return {(Res)obj};
             }
 
@@ -280,10 +245,10 @@ namespace gridtools {
              *  a proxy for sid_get_ptr_diff ADL resolution
              */
             template <class Sid, class Res = decltype(sid_get_ptr_diff(std::declval<Sid const &>()))>
-            std::enable_if_t<!std::is_same<Res, not_provided>::value, Res> get_ptr_diff(Sid const &);
+            std::enable_if_t<!std::is_same_v<Res, not_provided>, Res> get_ptr_diff(Sid const &);
 
             template <class Sid, class Res = decltype(sid_get_ptr_diff(std::declval<Sid const &>()))>
-            std::enable_if_t<std::is_same<Res, not_provided>::value, default_ptr_diff<ptr_type<Sid>>> get_ptr_diff(
+            std::enable_if_t<std::is_same_v<Res, not_provided>, default_ptr_diff<ptr_type<Sid>>> get_ptr_diff(
                 Sid const &);
 
             /**
@@ -300,13 +265,13 @@ namespace gridtools {
              *  `get_strides` delegates to `sid_get_strides`
              */
             template <class Sid, class Res = decltype(sid_get_strides(std::declval<Sid const &>()))>
-            std::enable_if_t<!std::is_same<Res, not_provided>::value && !std::is_array<Sid>::value, Res> get_strides(
+            std::enable_if_t<!std::is_same_v<Res, not_provided> && !std::is_array_v<Sid>, Res> get_strides(
                 Sid const &obj) {
                 return sid_get_strides(obj);
             }
 
             template <class Sid, class Res = decltype(sid_get_strides(std::declval<Sid const &>()))>
-            std::enable_if_t<std::is_same<Res, not_provided>::value && !std::is_array<Sid>::value, tuple<>> get_strides(
+            std::enable_if_t<std::is_same_v<Res, not_provided> && !std::is_array_v<Sid>, tuple<>> get_strides(
                 Sid const &) {
                 return {};
             }
@@ -326,7 +291,7 @@ namespace gridtools {
             struct get_array_strides<Inner[N], ElemSize> : get_array_strides<Inner[], ElemSize> {};
 
             template <class T>
-            std::enable_if_t<std::is_array<T>::value, typename get_array_strides<T>::type> get_strides(T const &) {
+            std::enable_if_t<std::is_array_v<T>, typename get_array_strides<T>::type> get_strides(T const &) {
                 return {};
             }
 
@@ -341,10 +306,10 @@ namespace gridtools {
             // BEGIN `strides_kind` PART
 
             template <class Sid, class Res = decltype(sid_get_strides_kind(std::declval<Sid const &>()))>
-            std::enable_if_t<!std::is_same<Res, not_provided>::value, Res> get_strides_kind(Sid const &);
+            std::enable_if_t<!std::is_same_v<Res, not_provided>, Res> get_strides_kind(Sid const &);
 
             template <class Sid, class Res = decltype(sid_get_strides_kind(std::declval<Sid const &>()))>
-            std::enable_if_t<std::is_same<Res, not_provided>::value, default_kind<strides_type<Sid>>> get_strides_kind(
+            std::enable_if_t<std::is_same_v<Res, not_provided>, default_kind<strides_type<Sid>>> get_strides_kind(
                 Sid const &);
 
             /**
@@ -359,45 +324,44 @@ namespace gridtools {
             // BEGIN `get_lower_bounds`/`get_upper_bounds`
 
             template <class Sid, class Res = decltype(sid_get_lower_bounds(std::declval<Sid const &>()))>
-            std::enable_if_t<!std::is_same<Res, not_provided>::value && !std::is_array<Sid>::value, Res>
-            get_lower_bounds(Sid const &obj) {
+            std::enable_if_t<!std::is_same_v<Res, not_provided> && !std::is_array_v<Sid>, Res> get_lower_bounds(
+                Sid const &obj) {
                 return sid_get_lower_bounds(obj);
             }
 
             template <class Sid, class Res = decltype(sid_get_upper_bounds(std::declval<Sid const &>()))>
-            std::enable_if_t<!std::is_same<Res, not_provided>::value && !std::is_array<Sid>::value, Res>
-            get_upper_bounds(Sid const &obj) {
+            std::enable_if_t<!std::is_same_v<Res, not_provided> && !std::is_array_v<Sid>, Res> get_upper_bounds(
+                Sid const &obj) {
                 return sid_get_upper_bounds(obj);
             }
 
             template <class Sid, class Res = decltype(sid_get_lower_bounds(std::declval<Sid const &>()))>
-            std::enable_if_t<std::is_same<Res, not_provided>::value && !std::is_array<Sid>::value, tuple<>>
-            get_lower_bounds(Sid const &) {
+            std::enable_if_t<std::is_same_v<Res, not_provided> && !std::is_array_v<Sid>, tuple<>> get_lower_bounds(
+                Sid const &) {
                 return {};
             }
 
             template <class Sid, class Res = decltype(sid_get_upper_bounds(std::declval<Sid const &>()))>
-            std::enable_if_t<std::is_same<Res, not_provided>::value && !std::is_array<Sid>::value, tuple<>>
-            get_upper_bounds(Sid const &) {
+            std::enable_if_t<std::is_same_v<Res, not_provided> && !std::is_array_v<Sid>, tuple<>> get_upper_bounds(
+                Sid const &) {
                 return {};
             }
 
-            template <class T,
-                class Res = meta::rename<tuple, meta::repeat<std::rank<T>, integral_constant<ptrdiff_t, 0>>>>
-            std::enable_if_t<std::is_array<T>::value, Res> get_lower_bounds(T const &) {
+            template <class T, class Res = meta::repeat<std::rank<T>, tuple<integral_constant<ptrdiff_t, 0>>>>
+            std::enable_if_t<std::is_array_v<T>, Res> get_lower_bounds(T const &) {
                 return {};
             }
 
             template <class T>
             struct array_extent_f {
                 template <class Dim>
-                using apply = integral_constant<ptrdiff_t, std::extent<T, Dim::value>::value>;
+                using apply = integral_constant<ptrdiff_t, std::extent_v<T, Dim::value>>;
             };
 
             template <class T,
                 class Dims = meta::make_indices<std::rank<T>, tuple>,
                 class Res = meta::transform<array_extent_f<T>::template apply, Dims>>
-            std::enable_if_t<std::is_array<T>::value, Res> get_upper_bounds(T const &) {
+            std::enable_if_t<std::is_array_v<T>, Res> get_upper_bounds(T const &) {
                 return {};
             }
 
@@ -421,8 +385,8 @@ namespace gridtools {
              *  If stride of offset are zero or the target has no state, we don't need to shift
              */
             template <class T, class Stride, class Offset>
-            using need_shift = bool_constant<!(std::is_empty<T>::value || is_integral_constant_of<Stride, 0>::value ||
-                                               is_integral_constant_of<Offset, 0>::value)>;
+            using need_shift = std::bool_constant<!(std::is_empty_v<T> || is_integral_constant_of<Stride, 0>::value ||
+                                                    is_integral_constant_of<Offset, 0>::value)>;
 
             /**
              *  additional proxy is used here to ensure that evaluation context of `obj += stride * offset`
@@ -439,7 +403,7 @@ namespace gridtools {
             template <class T, class Stride>
             struct is_default_shiftable<T,
                 Stride,
-                void_t<decltype(::gridtools::sid::concept_impl_::default_shift(
+                std::void_t<decltype(::gridtools::sid::concept_impl_::default_shift(
                     std::declval<T &>(), std::declval<Stride const &>()))>> : std::true_type {};
 
             template <class T>
@@ -451,7 +415,7 @@ namespace gridtools {
             template <class T, class = void>
             struct has_inc : std::false_type {};
             template <class T>
-            struct has_inc<T, void_t<decltype(::gridtools::sid::concept_impl_::inc_operator(std::declval<T &>()))>>
+            struct has_inc<T, std::void_t<decltype(::gridtools::sid::concept_impl_::inc_operator(std::declval<T &>()))>>
                 : std::true_type {};
 
             template <class T>
@@ -463,7 +427,7 @@ namespace gridtools {
             template <class T, class = void>
             struct has_dec : std::false_type {};
             template <class T>
-            struct has_dec<T, void_t<decltype(::gridtools::sid::concept_impl_::dec_operator(std::declval<T &>()))>>
+            struct has_dec<T, std::void_t<decltype(::gridtools::sid::concept_impl_::dec_operator(std::declval<T &>()))>>
                 : std::true_type {};
 
             template <class T, class Arg>
@@ -477,15 +441,16 @@ namespace gridtools {
             template <class T, class Arg>
             struct has_dec_assignment<T,
                 Arg,
-                void_t<decltype(::gridtools::sid::concept_impl_::dec_assignment_operator(
+                std::void_t<decltype(::gridtools::sid::concept_impl_::dec_assignment_operator(
                     std::declval<T &>(), std::declval<Arg const &>()))>> : std::true_type {};
 
             /**
              *  noop `shift` overload
              */
             template <class T, class Stride, class Offset>
-            GT_FUNCTION std::enable_if_t<!need_shift<T, Stride, Offset>::value> shift(
-                T &, Stride const &, Offset const &) {}
+            GT_FUNCTION std::enable_if_t<!(std::is_lvalue_reference_v<T> &&
+                                           need_shift<std::remove_reference_t<T>, std::decay_t<Stride>, Offset>::value)>
+            shift(T &&, Stride &&, Offset) {}
 
             /**
              * `shift` overload that delegates to `sid_shift`
@@ -499,7 +464,7 @@ namespace gridtools {
                                      !is_default_shiftable<T, std::decay_t<Stride>>::value,
                     int> = 0>
             GT_FUNCTION decltype(auto) shift(T &obj, Stride &&stride, Offset offset) {
-                return sid_shift(obj, wstd::forward<Stride>(stride), offset);
+                return sid_shift(obj, std::forward<Stride>(stride), offset);
             }
 
             /**
@@ -599,7 +564,7 @@ namespace gridtools {
                 !(has_dec_assignment<T, Decayed>::value &&
                     (is_integral_constant_of<Decayed, -1>::value || is_integral_constant_of<Offset, -1>::value))>
             shift(T &obj, Stride &&stride, Offset offset) {
-                obj += wstd::forward<Stride>(stride) * offset;
+                obj += std::forward<Stride>(stride) * offset;
             }
 
             // END `shift` PART
@@ -613,7 +578,7 @@ namespace gridtools {
                 struct apply : std::false_type {};
                 template <class Stride>
                 struct apply<Stride,
-                    void_t<decltype(::gridtools::sid::concept_impl_::shift(
+                    std::void_t<decltype(::gridtools::sid::concept_impl_::shift(
                         std::declval<T &>(), std::declval<Stride &>(), int_t{}))>> : std::true_type {};
             };
 
@@ -643,11 +608,11 @@ namespace gridtools {
                 class StridesKind = strides_kind<Sid>,
                 class LowerBoundsType = lower_bounds_type<Sid>,
                 class UpperBoundsType = upper_bounds_type<Sid>>
-            using is_sid = conjunction<
+            using is_sid = std::conjunction<
 
                 // `is_trivially_copyable` check is applied to the types that are will be passed from host to device
-                std::is_trivially_copyable<PtrHolder>,
-                std::is_trivially_copyable<StridesType>,
+                std::is_trivially_copy_constructible<PtrHolder>,
+                std::is_trivially_copy_constructible<StridesType>,
 
                 // verify that `PtrDiff` is sane
                 std::is_default_constructible<PtrDiff>,
@@ -661,7 +626,7 @@ namespace gridtools {
                     PtrHolder>,
 
                 // verify that `Reference` is sane
-                negation<std::is_void<ReferenceType>>,
+                std::negation<std::is_void<ReferenceType>>,
 
                 // all strides must be applied via `shift` with both `Ptr` and `PtrDiff`
                 are_valid_strides<StrideTypeList, Ptr>,
@@ -715,7 +680,7 @@ namespace gridtools {
          *  The const variation of the reference type
          */
         template <class Sid, class Ref = reference_type<Sid>>
-        using const_reference_type = std::conditional_t<std::is_reference<Ref>::value,
+        using const_reference_type = std::conditional_t<std::is_reference_v<Ref>,
             std::add_lvalue_reference_t<std::add_const_t<std::remove_reference_t<Ref>>>,
             std::add_const_t<Ref>>;
 
@@ -726,8 +691,8 @@ namespace gridtools {
          *  Which allows to silently ignore the offsets in non existing dimensions.
          */
         template <class Key, class Strides>
-        GT_CONSTEXPR GT_FUNCTION decltype(auto) get_stride(Strides &&strides) {
-            return gridtools::host_device::at_key_with_default<Key, default_stride>(wstd::forward<Strides>(strides));
+        constexpr GT_FUNCTION decltype(auto) get_stride(Strides &&strides) {
+            return gridtools::host_device::at_key_with_default<Key, default_stride>(std::forward<Strides>(strides));
         }
 
         /**
@@ -738,16 +703,16 @@ namespace gridtools {
             class Dim,
             class Strides,
             std::enable_if_t<has_key<std::decay_t<Strides>, Dim>::value, int> = 0>
-        GT_CONSTEXPR GT_FUNCTION decltype(auto) get_stride_element(Strides &&strides) {
+        constexpr GT_FUNCTION decltype(auto) get_stride_element(Strides &&strides) {
             return gridtools::host_device::at_key<Key>(
-                gridtools::host_device::at_key<Dim>(wstd::forward<Strides>(strides)));
+                gridtools::host_device::at_key<Dim>(std::forward<Strides>(strides)));
         }
 
         template <class Key,
             class Dim,
             class Strides,
             std::enable_if_t<!has_key<std::decay_t<Strides>, Dim>::value, int> = 0>
-        GT_CONSTEXPR GT_FUNCTION integral_constant<int_t, 0> get_stride_element(Strides &&) {
+        constexpr GT_FUNCTION integral_constant<int_t, 0> get_stride_element(Strides &&) {
             return {};
         }
 
@@ -757,14 +722,20 @@ namespace gridtools {
          *  If `Bounds` doesn't have `Key`, integral_constant<int_t, min/max> is returned.
          */
         template <class Key, class Bounds>
-        GT_CONSTEXPR GT_FUNCTION decltype(auto) get_lower_bound(Bounds &&bounds) {
+        decltype(auto) get_lower_bound(Bounds &&bounds) {
             return gridtools::host_device::at_key_with_default<Key,
-                integral_constant<int_t, std::numeric_limits<int_t>::min()>>(wstd::forward<Bounds>(bounds));
+                integral_constant<int_t, std::numeric_limits<int_t>::min()>>(std::forward<Bounds>(bounds));
         }
         template <class Key, class Bounds>
-        GT_CONSTEXPR GT_FUNCTION decltype(auto) get_upper_bound(Bounds &&bounds) {
+        decltype(auto) get_upper_bound(Bounds &&bounds) {
             return gridtools::host_device::at_key_with_default<Key,
-                integral_constant<int_t, std::numeric_limits<int_t>::max()>>(wstd::forward<Bounds>(bounds));
+                integral_constant<int_t, std::numeric_limits<int_t>::max()>>(std::forward<Bounds>(bounds));
+        }
+
+        template <class T, class Stride, class Offset>
+        GT_FUNCTION T shifted(T obj, Stride const &stride, Offset offset) {
+            shift(obj, stride, offset);
+            return obj;
         }
     } // namespace sid
 
@@ -774,4 +745,9 @@ namespace gridtools {
      *  Just because `sid::is_sid` looks a bit redundant
      */
     using sid::is_sid;
+
+#ifdef __cpp_concepts
+    template <class T>
+    concept Sid = is_sid<T>::value;
+#endif
 } // namespace gridtools
