@@ -16,15 +16,18 @@ function module() {
 }
 
 module load daint-gpu
-module load cudatoolkit/10.1.105_3.27-7.0.1.1_4.1__ga311ce7
-module rm CMake
-module load /users/jenkins/easybuild/daint/haswell/modules/all/CMake/3.14.5
+module load cudatoolkit
 
-export BOOST_ROOT=/project/c14/install/daint/boost/boost_1_67_0/
+export EASYBUILD_PREFIX=/apps/daint/SSL/gridtools/jenkins/easybuild
+module use $EASYBUILD_PREFIX/modules/all
+module load CMake
+
+export BOOST_ROOT=/apps/daint/SSL/gridtools/jenkins/boost_1_77_0
 export CUDATOOLKIT_HOME=$CUDA_PATH
 export CUDA_ARCH=sm_60
 
 export GTRUN_BUILD_COMMAND='srun -C gpu -p cscsci --time=00:20:00 make -j 24'
+export GTRUN_SBATCH_ACCOUNT='d75'
 export GTRUN_SBATCH_PARTITION='cscsci'
 export GTRUN_SBATCH_NODES=1
 export GTRUN_SBATCH_NTASKS_PER_CORE=2
